@@ -53,6 +53,8 @@ appDependencies <- function(project = NULL,
     .packrat_mutables$origLibPaths
   )
 
+  ignores <- packrat::opts$ignored.packages()
+
   ## For R packages, we only use the DESCRIPTION file
   if (isRPackage(project)) {
 
@@ -61,20 +63,22 @@ appDependencies <- function(project = NULL,
       pkgDescriptionDependencies(file.path(project, "DESCRIPTION"))$Package
 
     # Strip out any dependencies the user has requested we do not track.
-    parentDeps <- setdiff(parentDeps, packrat::opts$ignored.packages())
+    parentDeps <- setdiff(parentDeps, ignores)
 
     ## For downstream dependencies, we don't grab their Suggests:
     ## Presumedly, we can build child dependencies without vignettes, and hence
     ## do not need suggests -- for the package itself, we should make sure
     ## we grab suggests, however
     childDeps <- recursivePackageDependencies(parentDeps,
+                                              ignores,
                                               libPaths,
                                               available.packages,
                                               fields)
   } else {
     parentDeps <- setdiff(unique(c(dirDependencies(project))), "packrat")
-    parentDeps <- setdiff(parentDeps, packrat::opts$ignored.packages())
+    parentDeps <- setdiff(parentDeps, ignores)
     childDeps <- recursivePackageDependencies(parentDeps,
+                                              ignores,
                                               libPaths,
                                               available.packages,
                                               fields)
@@ -106,10 +110,84 @@ appDependencies <- function(project = NULL,
 
 # detect all package dependencies for a directory of files
 dirDependencies <- function(dir) {
+  if (as.logical(getOption("packrat.dependency.discovery.disabled", default = FALSE))) {
+    character()
+  } else if (as.logical(getOption("packrat.dependency.discovery.renv", default = TRUE))) {
+    dirDependenciesRenv(dir)
+  } else {
+    dirDependenciesBuiltIn(dir)
+  }
+}
+
+# Return renv ignore patterns based on the packrat ignored.directories option.
+# Each directory is returned as a rooted pattern for renv, meaning that it
+# should only apply at the root directory of the project.
+#
+# Note: The "/data/" and "/inst/" directories are ignored by default.
+#
+# See: https://github.com/rstudio/renv/pull/866
+ignoresForRenv <- function(ignoredDirectories) {
+  ignores <- NULL
+  if (length(ignoredDirectories) > 0) {
+    ignores <- ignoredDirectories
+    # Make sure all the directories end with a slash.
+    ignores <- ifelse(
+        substr(ignores, nchar(ignores), nchar(ignores)) != "/",
+        paste0(ignores, "/"),
+        ignores
+    )
+    # Make sure all the directories begin with a slash.
+    ignores <- ifelse(
+        substr(ignores, 1, 1) != "/",
+        paste0("/", ignores),
+        ignores
+    )
+    attr(ignores, "asis") <- TRUE
+  }
+  ignores
+}
+
+dirDependenciesRenv <- function(dir) {
+  old_filebacked_cache <- options(renv.config.filebacked.cache = FALSE)
+  on.exit(do.call(options, old_filebacked_cache), add = TRUE)
+
+  project <- Sys.getenv("RENV_PROJECT", unset = NA)
+  if (!is.na(project)) {
+    Sys.unsetenv("RENV_PROJECT")
+    on.exit(Sys.setenv(RENV_PROJECT = project), add = TRUE)
+  }
+
+  profile <- Sys.getenv("RENV_PROFILE", unset = NA)
+  if (!is.na(profile)) {
+    Sys.unsetenv("RENV_PROFILE")
+    on.exit(Sys.setenv(RENV_PROFILE = profile), add = TRUE)
+  }
+
+  old_ignored_packages <- options("renv.settings.ignored.packages" = opts$ignored.packages())
+  on.exit(do.call(options, old_ignored_packages), add = TRUE)
+
+  old_renv_exclude <- options("renv.renvignore.exclude" = ignoresForRenv(opts$ignored.directories()))
+  on.exit(do.call(options, old_renv_exclude), add = TRUE)
+
+  # TODO: add rsconnect as an ignored directory? May not be an issue for
+  # bundling, since we don't include the rsconnect directory.
+
+  absDir <- normalizePath(dir)
+  deps <- renv$dependencies(path = absDir, root = absDir, quiet = TRUE)
+  pkgs <- unique(deps$Package)
+  ## Exclude recommended packages (and the artifical "R" package) if there is
+  ## no package installed locally this places an implicit dependency on the
+  ## system-installed version of a package
+  pkgs <- dropSystemPackages(pkgs)
+  pkgs
+}
+
+# detect all package dependencies for a directory of files
+dirDependenciesBuiltIn <- function(dir) {
   dir <- normalizePath(dir, winslash = '/')
 
   # first get the packages referred to in source code
-  pattern <- "[.](?:r|rmd|rnw|rpres)$"
+  pattern <- "[.](?:r|rmd|qmd|rnw|rpres)$"
   pkgs <- character()
   R_files <- list.files(dir,
                         pattern = pattern,
@@ -144,13 +222,11 @@ dirDependencies <- function(dir) {
     R_files <- grep(ignoredDirRegex, R_files, invert = TRUE, value = TRUE)
   }
 
-  if (!identical(getOption("packrat.dependency.discovery.disabled"), TRUE)) {
-    sapply(R_files, function(file) {
-      filePath <- file.path(dir, file)
-      pkgs <<- append(pkgs, fileDependencies(file.path(dir, file)))
+  sapply(R_files, function(file) {
+    filePath <- file.path(dir, file)
+    pkgs <<- append(pkgs, fileDependencies(file.path(dir, file)))
 
-    })
-  }
+  })
 
   ## Exclude recommended packages if there is no package installed locally
   ## this places an implicit dependency on the system-installed version of a package
@@ -168,6 +244,7 @@ fileDependencies <- function(file) {
   switch(fileext,
          r = fileDependencies.R(file),
          rmd = fileDependencies.Rmd(file),
+         qmd = fileDependencies.Qmd(file),
          rnw = fileDependencies.Rnw(file),
          rpres = fileDependencies.Rpres(file),
          stop("Unrecognized file type '", file, "'")
@@ -180,10 +257,12 @@ hasYamlFrontMatter <- function(content) {
 }
 
 yamlDeps <- function(yaml) {
-  c(
+  unique(c(
     "shiny"[any(grepl("runtime:[[:space:]]*shiny", yaml, perl = TRUE))],
+    "shiny"[any(grepl("server:[[:space:]]*shiny", yaml, perl = TRUE))],
+    "shiny"[any(grepl("[[:space:]]+type:[[:space:]]*shiny", yaml, perl = TRUE))],
     "rticles"[any(grepl("rticles::", yaml, perl = TRUE))]
-  )
+  ))
 }
 
 stripAltEngines <- function(file, encoding) {
@@ -208,9 +287,27 @@ stripAltEngines <- function(file, encoding) {
   writeLines(contents[regions], file)
 }
 
-fileDependencies.Rmd <- function(file) {
+# compute package dependencies for an *.qmd file. not all Quarto documents
+# require R/rmarkdown.
+#
+# Quarto/rsconnect may independently indicate that this file needs the knitr
+# engine and will communicate an implicit dependency on rmarkdown
+fileDependencies.Qmd <- function(file) {
+  fileDependencies.Markdown(file, implicit = NULL)
+}
 
-  deps <- "rmarkdown"
+# compute package dependencies for an *.Rmd file. rmarkdown is an automatic,
+# implicit dependency.
+fileDependencies.Rmd <- function(file) {
+  fileDependencies.Markdown(file, implicit = c("rmarkdown"))
+}
+
+fileDependencies.Markdown <- function(file, implicit = NULL) {
+
+  deps <- c()
+  if (!is.null(implicit)) {
+    deps <- c(deps, implicit)
+  }
 
   # try using an evaluate-based approach for dependencies
   if (knitrHasEvaluateHook()) {
@@ -229,7 +326,7 @@ fileDependencies.Rmd <- function(file) {
       }
 
       # render with a custom evaluate hook to discover dependencies
-      deps <- c(deps, fileDependencies.Rmd.evaluate(file))
+      deps <- c(deps, fileDependencies.evaluate(file))
     }
   }
 
@@ -313,7 +410,7 @@ fileDependencies.Rmd <- function(file) {
   if (requireNamespace("knitr", quietly = TRUE)) {
     deps <- c(
       deps,
-      fileDependencies.Rmd.tangle(file, encoding = encoding)
+      fileDependencies.tangle(file, encoding = encoding)
     )
   } else {
     warning("knitr is required to parse dependencies but is not available")
@@ -390,7 +487,7 @@ allOf <- function(object, ...) {
 
 recursiveWalk <- function(`_node`, fn, ...) {
   fn(`_node`, ...)
-  if (is.call(`_node`)) {
+  if (is.recursive(`_node`)) {
     for (i in seq_along(`_node`)) {
       recursiveWalk(`_node`[[i]], fn, ...)
     }
@@ -471,7 +568,6 @@ identifyPackagesUsed <- function(call, env) {
 }
 
 expressionDependencies <- function(e) {
-
   if (is.expression(e)) {
     return(unlist(lapply(e, function(call) {
       expressionDependencies(call)
@@ -572,11 +668,25 @@ knitrHasEvaluateHook <- function() {
 }
 
 
-fileDependencies.Rmd.evaluate <- function(file) {
+fileDependencies.evaluate <- function(file) {
 
   # discovered packages (to be updated by evaluate hook)
   deps <- list()
 
+  # override any existing engines -- we don't want dependency discovery
+  # to, say, run arbitrary bash scripts contained in the document!
+  engines <- knitr::knit_engines$get()
+  on.exit(knitr::knit_engines$restore(engines), add = TRUE)
+
+  # generate overrides
+  overrides <- replicate(length(engines), function(options) {}, FALSE)
+  names(overrides) <- names(engines)
+
+  # retain the regular R knitr hook, and treat Rscript chunks
+  # the same way as "regular" R chunks
+  overrides$R <- overrides$Rscript <- engines$R
+  knitr::knit_engines$set(overrides)
+
   # save old hook and install our custom hook
   evaluate_hook <- knitr::knit_hooks$get("evaluate")
   on.exit(knitr::knit_hooks$set(evaluate = evaluate_hook), add = TRUE)
@@ -587,6 +697,11 @@ fileDependencies.Rmd.evaluate <- function(file) {
     })
   })
 
+  # keep going on error
+  chunkOptions <- knitr::opts_chunk$get()
+  on.exit(knitr::opts_chunk$restore(chunkOptions), add = TRUE)
+  knitr::opts_chunk$set(error = TRUE)
+
   # rudely override knitr's 'inline_exec' function so
   # that we can detect dependencies within inline chunks
   knitr <- asNamespace("knitr")
@@ -616,12 +731,28 @@ fileDependencies.Rmd.evaluate <- function(file) {
   }
 
   # attempt to render document with our custom hook active
+  # TODO: do we want to report errors here? right now we're just
+  # capturing and silently discarding render errors
   outfile <- tempfile()
+  on.exit(unlink(outfile), add = TRUE)
+
   tryCatch(
-    rmarkdown::render(file, output_file = outfile, quiet = TRUE),
+    withCallingHandlers(
+      rmarkdown::render(file, output_file = outfile, quiet = TRUE),
+      warning = function(w) {
+
+        # ignore warnings emitted by knitr::get_engine()
+        get_engine <- yoink("knitr", "get_engine")
+        for (i in seq_len(sys.nframe())) {
+          fn <- sys.function(i)
+          if (identical(fn, get_engine))
+            invokeRestart("muffleWarning")
+        }
+
+      }
+    ),
     error = identity
   )
-  unlink(outfile)
 
   unique(unlist(deps, recursive = TRUE))
 }
@@ -632,7 +763,7 @@ fileDependencies.Rmd.evaluate <- function(file) {
 # Extract dependencies per chunk rather than per file.
 # Packages like learnr have special R code chunks that are not evaluated at run time.
 # While the .Rmd file can be rendered with rmarkdown, a raw tangled R file may not be able to be processed.
-fileDependencies.Rmd.tangle <- function(file, encoding = "UTF-8") {
+fileDependencies.tangle <- function(file, encoding = "UTF-8") {
 
   # discovered packages
   deps <- list()
